==> galeon-0.11.3.tar.gz: galeon-0.11.3/src/downloader.c <==
/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Galeon includes */
#include "galeon.h"
#include "Gtm.h"
#include "mozilla.h"
#include "downloader.h"
#include "filepicker.h"

/* GNOME includes */
#include <libgnorba/gnorba.h>

static gboolean downloader_gtm_add_url (const gchar *url, gchar *dir, 
					gboolean disable_proxy,
					gboolean disable_auto_dl);
static void downloader_save_url_with_dir (const gchar *url, 
					  const char *directory);
static void downloader_save_url_without_dir (const gchar *url);
static void downloader_save_url_with_command_line (const gchar *url, 
						   const gchar *command,
						   const gchar *directory);
static void downloader_save_url_with_gtm (const gchar *url, 
					  const gchar *directory);

/**
 * save_link:
 */
void
downloader_save_link (GaleonEmbed *embed, const gchar *url)
{
	switch (gnome_config_get_int (CONF_DOWNLOADING_DOWNLOADER))
	{
	case DOWNLOADER_BUILTIN:
		mozilla_save_link (embed, url);
		break;
		
	case DOWNLOADER_GTM:
	case DOWNLOADER_EXTERNAL:
		downloader_save_url (url);
		break;
	}
}

/**
 * save_url
 */
void
downloader_save_url (const gchar *url)
{
	if (gnome_config_get_bool (CONF_DOWNLOADING_ASK_DIR))
	{
		gchar *dirName, *retPath = NULL;
		gboolean ret;
		dirName = gnome_config_get_string (CONF_DOWNLOADING_DIR);
		if (!dirName)
		{
			dirName = g_strdup (g_get_home_dir());
		}
		ret = show_file_picker (NULL, N_("Choose destination folder"),
				        dirName, NULL, modeGetFolder, &retPath);
		if (ret == TRUE)
		{
			downloader_save_url_with_dir (url, retPath);
		}
		g_free (dirName);
		g_free (retPath);
	}
	else
	{
		downloader_save_url_without_dir (url);
	}
}

static void
downloader_save_url_with_dir (const gchar *url, const char *directory)
{
	Downloader downloader;
	gchar *command;

	/* which downloader? */
	downloader = gnome_config_get_int (CONF_DOWNLOADING_DOWNLOADER);
	if (downloader == DOWNLOADER_EXTERNAL)
	{
		/* get command */
		command = gnome_config_get_string 
			(CONF_DOWNLOADING_EXTERNAL_COMMAND);

		/* check available */
		if (command != NULL)
		{
			/* check valid */
			if (strlen (command) > 0)
			{
				downloader_save_url_with_command_line 
					(url, command, directory);
				return;
			}

			/* free allocated string */
			g_free (command);
		}
	}

	/* fallthrough default */
	downloader_save_url_with_gtm (url, directory);
}

static void
downloader_save_url_without_dir (const gchar *url)
{
	Downloader downloader;
	gchar *command, *dir;

	/* get directory to save to */
	dir = gnome_config_get_string (CONF_DOWNLOADING_DIR);

	/* which downloader? */
	downloader = gnome_config_get_int (CONF_DOWNLOADING_DOWNLOADER);
	if (downloader == DOWNLOADER_EXTERNAL)
	{
		/* get command */
		command = gnome_config_get_string 
			(CONF_DOWNLOADING_EXTERNAL_COMMAND);

		/* check available */
		if (command != NULL)
		{
			/* check valid */
			if (strlen (command) > 0)
			{
				downloader_save_url_with_command_line 
					(url, command, dir);
				g_free (command);
				g_free (dir);
				return;
			}

			/* free allocated string */
			g_free (command);
		}
	}

	/* fallthrough default */
	downloader_save_url_with_gtm (url, dir);
	g_free (dir);
}

static void 
downloader_save_url_with_command_line (const gchar *url, const gchar *command,
				       const gchar *directory)
{
	GtkWidget *dialog;
	gchar *full_command;
	gchar *text;	
	gint pid;

	full_command = g_strdup_printf (command, url);
	
	if (gnome_config_get_bool (CONF_DOWNLOADING_EXTERNAL_TERMINAL))
	{
		char *argv[5] = { "gnome-terminal", "-t", 
				  _("Galeon Downloader"), "-e",
				 full_command };
		
		/* FIXME: this will only detect if the execution
		 * of gnome-terminal failed... */
		pid = gnome_execute_async (directory, 5, argv);	
	}
	else
	{
		int i;
		gchar **argv = g_strsplit (full_command, " ", 100);
		for (i=0; argv[i] != NULL; i++);

		pid = gnome_execute_async (directory, i, argv);
		g_strfreev (argv);
	}
	if (pid == -1)
	{
		text = g_strdup_printf (_("Failed to execute download "
					  "command:\n\"%s\""), full_command);
		dialog = gnome_error_dialog (text);
		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
		g_free (text);
	}

	g_free (full_command);
}

/**
 * save_url_with_gtm
 */
static void
downloader_save_url_with_gtm (const gchar *url, const gchar *directory)
{  
	gboolean auto_download;

	auto_download = gnome_config_get_bool (CONF_DOWNLOADING_AUTO_DOWNLOAD);
	downloader_gtm_add_url (strdup(url), (char *)directory,
				FALSE, !auto_download);
}

/*
 * Calls the gtm CORBA interface function gtm_add_url() to place the
 * download in queue
 */
static gboolean 
downloader_gtm_add_url (const gchar *url, gchar *dir, gboolean disable_proxy,
			gboolean disable_auto_dl)
{
	CORBA_Environment ev;
	GTM_Download gtm_download_client;
	static gboolean gnorba_init = FALSE;
	gboolean result;

	/* exception catching environment */
	CORBA_exception_init (&ev);

	/* initialise gnorba */
	if (!galeon_panel_mode && !gnorba_init)
	{
		/* use dummy environment */
		int argc = 0;
		char *argv[1] = { NULL };
		gnorba_CORBA_init (&argc, argv, GNORBA_INIT_SERVER_FUNC, &ev);
		gnorba_init = TRUE;
	}

	result = FALSE;
	gtm_download_client = 
	goad_server_activate_with_repo_id (NULL, "IDL:gtm/download:1.0",
					   0, NULL);

	if (gtm_download_client != NULL)
	{
		result = GTM_Download_add_url ( gtm_download_client, url, dir,
						disable_proxy,
						disable_auto_dl, &ev);
		
		if (ev._major != CORBA_NO_EXCEPTION)
		{
			g_warning ("Exception thrown: %s\n", 
				   CORBA_exception_id (&ev));
			CORBA_exception_free (&ev);
		}
	}
	else
	{
		GtkWidget *dialog;
		gchar *text;

		text = g_strdup_printf (_("Failed to communicate with GTM.\n\n"
					  "Please check it is installed "
					  "correctly. Download it from:\n"
					  "\thttp://gtm.sourceforge.net"));
		dialog = gnome_error_dialog (text);
		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
		g_free (text);
	}
  
	return result;
}

==> galeon-0.12.tar.gz: galeon-0.12/src/downloader.c <==
/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Galeon includes */
#include "galeon.h"
#include "Gtm.h"
#include "mozilla.h"
#include "downloader.h"
#include "filepicker.h"
#include "prefs.h"

/* GNOME includes */
#include <libgnorba/gnorba.h>
#include <libgnomevfs/gnome-vfs-utils.h>

static gboolean downloader_gtm_add_url (const gchar *url, gchar *dir, 
					gboolean disable_proxy,
					gboolean disable_auto_dl);
static void downloader_save_url_with_dir (const gchar *url, 
					  const char *directory);
static void downloader_save_url_without_dir (const gchar *url);
static void downloader_save_url_with_command_line (const gchar *url, 
						   const gchar *command,
						   const gchar *directory);
static void downloader_save_url_with_gtm (const gchar *url, 
					  const gchar *directory);

/**
 * save_link:
 */
void
downloader_save_link (GaleonEmbed *embed, const gchar *url)
{
	switch (eel_gconf_get_integer (CONF_DOWNLOADING_DOWNLOADER))
	{
	case DOWNLOADER_BUILTIN:
		mozilla_save_link (embed, url);
		break;
		
	case DOWNLOADER_GTM:
	case DOWNLOADER_EXTERNAL:
		downloader_save_url (url);
		break;
	}
}

/**
 * save_url
 */
void
downloader_save_url (const gchar *url)
{
	if (eel_gconf_get_boolean (CONF_DOWNLOADING_ASK_DIR))
	{
		gchar *dirName, *retPath = NULL;
		gboolean ret;

		dirName = eel_gconf_get_string (CONF_DOWNLOADING_DIR);
		if (!dirName)
		{
			dirName = g_strdup (g_get_home_dir());
		}

		ret = show_file_picker (NULL, _("Choose destination folder"),
				        dirName, NULL, modeGetFolder, &retPath,
				        NULL);
		if (ret == TRUE)
		{
			downloader_save_url_with_dir (url, retPath);
		}
		g_free (dirName);
		g_free (retPath);
	}
	else
	{
		downloader_save_url_without_dir (url);
	}
}

static void
downloader_save_url_with_dir (const gchar *url, const char *directory)
{
	Downloader downloader;
	gchar *command;

	/* which downloader? */
	downloader = eel_gconf_get_integer (CONF_DOWNLOADING_DOWNLOADER);
	if (downloader == DOWNLOADER_EXTERNAL)
	{
		/* get command */
		command = eel_gconf_get_string 
			(CONF_DOWNLOADING_EXTERNAL_COMMAND);

		/* check available */
		if (command != NULL)
		{
			/* check valid */
			if (strlen (command) > 0)
			{
				downloader_save_url_with_command_line 
					(url, command, directory);
				return;
			}

			/* free allocated string */
			g_free (command);
		}
	}

	/* fallthrough default */
	downloader_save_url_with_gtm (url, directory);
}

static void
downloader_save_url_without_dir (const gchar *url)
{
	Downloader downloader;
	gchar *command, *dir, *exp_dir;

	/* get directory to save to */
	dir = eel_gconf_get_string (CONF_DOWNLOADING_DIR);
	if (dir)
	{
		exp_dir = gnome_vfs_expand_initial_tilde (dir);
		g_free (dir);
	}
	else
	{
		exp_dir = g_strdup (g_get_home_dir());
	}

	/* which downloader? */
	downloader = eel_gconf_get_integer (CONF_DOWNLOADING_DOWNLOADER);
	if (downloader == DOWNLOADER_EXTERNAL)
	{
		/* get command */
		command = eel_gconf_get_string 
		  (CONF_DOWNLOADING_EXTERNAL_COMMAND);

		/* check available */
		if (command != NULL)
		{
			/* check valid */
			if (strlen (command) > 0)
			{
				downloader_save_url_with_command_line 
					(url, command, exp_dir);
				g_free (command);
				g_free (exp_dir);
				return;
			}

			/* free allocated string */
			g_free (command);
		}
	}

	/* fallthrough default */
	downloader_save_url_with_gtm (url, exp_dir);

	g_free (exp_dir);
}

static void 
downloader_save_url_with_command_line (const gchar *url, const gchar *command,
				       const gchar *directory)
{
	GtkWidget *dialog;
	gchar *full_command;
	gchar *text;	
	gint pid;

	full_command = g_strdup_printf (command, url);
	
	if (eel_gconf_get_boolean (CONF_DOWNLOADING_EXTERNAL_TERMINAL))
	{
		char *argv[5] = { "gnome-terminal", "-t", 
				  _("Galeon Downloader"), "-e",
				 full_command };
		
		/* FIXME: this will only detect if the execution
		 * of gnome-terminal failed... */
		pid = gnome_execute_async (directory, 5, argv);	
	}
	else
	{
		int i;
		gchar **argv = g_strsplit (full_command, " ", 100);
		for (i=0; argv[i] != NULL; i++);

		pid = gnome_execute_async (directory, i, argv);
		g_strfreev (argv);
	}
	if (pid == -1)
	{
		text = g_strdup_printf (_("Failed to execute download "
					  "command:\n\"%s\""), full_command);
		dialog = gnome_error_dialog (text);
		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
		g_free (text);
	}

	g_free (full_command);
}

/**
 * save_url_with_gtm
 */
static void
downloader_save_url_with_gtm (const gchar *url, const gchar *directory)
{  
	gboolean auto_download;

	auto_download = eel_gconf_get_boolean (CONF_DOWNLOADING_AUTO_DOWNLOAD);
	downloader_gtm_add_url (strdup(url), (char *)directory,
				FALSE, !auto_download);
}

/*
 * Calls the gtm CORBA interface function gtm_add_url() to place the
 * download in queue
 */
static gboolean 
downloader_gtm_add_url (const gchar *url, gchar *dir, gboolean disable_proxy,
			gboolean disable_auto_dl)
{
	CORBA_Environment ev;
	GTM_Download gtm_download_client;
	static gboolean gnorba_init = FALSE;
	gboolean result;

	/* exception catching environment */
	CORBA_exception_init (&ev);

	/* initialise gnorba */
	if (!galeon_panel_mode && !gnorba_init)
	{
		/* use dummy environment */
		int argc = 0;
		char *argv[1] = { NULL };
		gnorba_CORBA_init (&argc, argv, GNORBA_INIT_SERVER_FUNC, &ev);
		gnorba_init = TRUE;
	}

	result = FALSE;
	gtm_download_client = 
	goad_server_activate_with_repo_id (NULL, "IDL:gtm/download:1.0",
					   0, NULL);

	if (gtm_download_client != NULL)
	{
		result = GTM_Download_add_url ( gtm_download_client, url, dir,
						disable_proxy,
						disable_auto_dl, &ev);
		
		if (ev._major != CORBA_NO_EXCEPTION)
		{
			g_warning ("Exception thrown: %s\n", 
				   CORBA_exception_id (&ev));
			CORBA_exception_free (&ev);
		}
	}
	else
	{
		GtkWidget *dialog;
		gchar *text;

		text = g_strdup_printf (_("Failed to communicate with GTM.\n\n"
					  "Please check it is installed "
					  "correctly. Download it from:\n"
					  "\thttp://gtm.sourceforge.net"));
		dialog = gnome_error_dialog (text);
		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
		g_free (text);
	}
  
	return result;
}
